@@ -55,10 +55,11 @@ def auto_move_data(fn: Callable) -> Callable:
     @wraps(fn)
     def auto_transfer_args(self, *args, **kwargs):
         from pytorch_lightning.core.lightning import LightningModule
+
         if not isinstance(self, LightningModule):
             return fn(self, *args, **kwargs)
 
-        args, kwargs = self.transfer_batch_to_device((args, kwargs))
+        args, kwargs = self.transfer_batch_to_device((args, kwargs), device=self.device, dataloader_idx=None)
         return fn(self, *args, **kwargs)
 
     rank_zero_deprecation(
@@ -96,10 +97,10 @@ def parameter_validation(fn: Callable) -> Callable:
 
         if not pre_layer_count == post_layer_count:
             rank_zero_warn(
-                f'The model layers do not match after moving to the target device.'
-                ' If your model employs weight sharing on TPU,'
-                ' please tie your weights using the `on_post_move_to_device` model hook.\n'
-                f'Layer count: [Before: {pre_layer_count} After: {post_layer_count}]'
+                f"The model layers do not match after moving to the target device."
+                " If your model employs weight sharing on TPU,"
+                " please tie your weights using the `on_post_move_to_device` model hook.\n"
+                f"Layer count: [Before: {pre_layer_count} After: {post_layer_count}]"
             )
 
         return module
