@@ -19,7 +19,6 @@ import logging
 import os
 from argparse import Namespace
 from copy import deepcopy
-from functools import partial
 from typing import Any, Callable, Dict, IO, MutableMapping, Optional, Union
 from warnings import warn
 
@@ -42,16 +41,13 @@ if _OMEGACONF_AVAILABLE:
     from omegaconf.errors import UnsupportedValueType, ValidationError
 
 # the older shall be on the top
-CHECKPOINT_PAST_HPARAMS_KEYS = (
-    'hparams',
-    'module_arguments',  # used in 0.7.6
-)
+CHECKPOINT_PAST_HPARAMS_KEYS = ("hparams", "module_arguments")  # used in 0.7.6
 
 
-class ModelIO(object):
-    CHECKPOINT_HYPER_PARAMS_KEY = 'hyper_parameters'
-    CHECKPOINT_HYPER_PARAMS_NAME = 'hparams_name'
-    CHECKPOINT_HYPER_PARAMS_TYPE = 'hparams_type'
+class ModelIO:
+    CHECKPOINT_HYPER_PARAMS_KEY = "hyper_parameters"
+    CHECKPOINT_HYPER_PARAMS_NAME = "hparams_name"
+    CHECKPOINT_HYPER_PARAMS_TYPE = "hparams_type"
 
     @classmethod
     def load_from_checkpoint(
@@ -135,15 +131,15 @@ class ModelIO(object):
             checkpoint = pl_load(checkpoint_path, map_location=lambda storage, loc: storage)
 
         if hparams_file is not None:
-            extension = hparams_file.split('.')[-1]
-            if extension.lower() == 'csv':
+            extension = hparams_file.split(".")[-1]
+            if extension.lower() == "csv":
                 hparams = load_hparams_from_tags_csv(hparams_file)
-            elif extension.lower() in ('yml', 'yaml'):
+            elif extension.lower() in ("yml", "yaml"):
                 hparams = load_hparams_from_yaml(hparams_file)
             else:
-                raise ValueError('.csv, .yml or .yaml is required for `hparams_file`')
+                raise ValueError(".csv, .yml or .yaml is required for `hparams_file`")
 
-            hparams['on_gpu'] = False
+            hparams["on_gpu"] = False
 
             # overwrite hparams by the given file
             checkpoint[cls.CHECKPOINT_HYPER_PARAMS_KEY] = hparams
@@ -202,7 +198,17 @@ class ModelIO(object):
         model.on_load_checkpoint(checkpoint)
 
         # load the state_dict on the model automatically
-        model.load_state_dict(checkpoint['state_dict'], strict=strict)
+        keys = model.load_state_dict(checkpoint["state_dict"], strict=strict)
+
+        if not strict:
+            if keys.missing_keys:
+                rank_zero_warn(
+                    f"Found keys that are in the model state dict but not in the checkpoint: {keys.missing_keys}"
+                )
+            if keys.unexpected_keys:
+                rank_zero_warn(
+                    f"Found keys that are not in the model state dict but in the checkpoint: {keys.unexpected_keys}"
+                )
 
         return model
 
@@ -381,8 +387,7 @@ def save_hparams_to_yaml(config_yaml, hparams: Union[dict, Namespace]) -> None:
     if _OMEGACONF_AVAILABLE:
         # deepcopy: hparams from user shouldn't be resolved
         hparams = deepcopy(hparams)
-        to_container = partial(OmegaConf.to_container, resolve=True)
-        hparams = apply_to_collection(hparams, DictConfig, to_container)
+        hparams = apply_to_collection(hparams, DictConfig, OmegaConf.to_container, resolve=True)
         with fs.open(config_yaml, "w", encoding="utf-8") as fp:
             try:
                 OmegaConf.save(hparams, fp)
