@@ -14,17 +14,15 @@
 import torch
 from torchmetrics.functional import embedding_similarity as _embedding_similarity
 
-from pytorch_lightning.metrics.utils import deprecated_metrics
+from pytorch_lightning.metrics.utils import deprecated_metrics, void
 
 
 @deprecated_metrics(target=_embedding_similarity)
 def embedding_similarity(
-    batch: torch.Tensor,
-    similarity: str = 'cosine',
-    reduction: str = 'none',
-    zero_diagonal: bool = True
+    batch: torch.Tensor, similarity: str = "cosine", reduction: str = "none", zero_diagonal: bool = True
 ) -> torch.Tensor:
     """
     .. deprecated::
         Use :func:`torchmetrics.functional.embedding_similarity`. Will be removed in v1.5.0.
     """
+    return void(batch, similarity, reduction, zero_diagonal)
